use anyhow::anyhow;
use async_trait::async_trait;

use crate::vault::{Config, VaultRef};

use self::c1::C1;

pub mod c1;

#[async_trait]
pub trait Challenge
where
    Self: Sized,
{
    fn new(config: Config, vault: VaultRef) -> Self;
    fn hint() -> String;
    async fn serve(self) -> anyhow::Result<()>;
    fn get_challenge(idx: u16, config: Config, vault: VaultRef) -> anyhow::Result<impl Challenge> {
        match idx {
            1 => Ok(C1::new(config, vault)),
            _ => Err(anyhow!("no challenge with that index exists")),
        }
    }
}
